/*
** DESCRIPTION: Example for the different loop scheduling strategies
** COMPILE:     g++ -fopenmp -o loops loops.c
** RUN:         export OMP_NUM_THREADS=4
**              export OMP_SCHEDULE="static"
**              time ./loops
**
** Try this program with different values for the environment variable
** OMP_SCHEDULE.
**  - E.g., "static,1", "dynamic", "dynamic,100", "guided"
**  - How does the execution time change? Why?
**
** If you uncomment the cout statement and sort the output using
**   ./loops | sort -n | more
** you can see which thread executes which loop iterations!
*/

#include <iostream>
#include <math.h>
#include <omp.h>

using namespace std;

main()
{
	int i, j;
	double x;

	#pragma omp parallel for private(i,j,x) schedule(runtime)
	for (i=0; i<40000; i++) {
		//cout << i << ": Thread " << omp_get_thread_num() << "\n";
		x = 1.2;
		for (j=0; j<i; j++) {
			x = sqrt(x) * sin(x*x);
		}
	}
}
