/*
** DESCRIPTION: Examples for sections and barriers.
** COMPILE:     g++ -fopenmp -o sections sections.c
** RUN:         export OMP_NUM_THREADS=2
**              ./sections
**
** Try the program with 2 and 3 threads!
**  - How does the execution change?
**
** Try adding the option 'nowait' to the first 'sections' directive and
** execute the program again with 2 and 3 threads!
**  - How does the execution change now?
*/

#include <iostream>
#include <unistd.h>
#include <omp.h>

using namespace std;

int main()
{
	int p;
    #pragma omp parallel private(p)
	{
		int thread = omp_get_thread_num();
	    #pragma omp sections nowait
		{
			#pragma omp section 
			{
				#pragma omp critical
				cout << "Thread " << thread << ", Section 1 start\n";
				usleep(200000);
				#pragma omp critical
				cout << "Thread " << thread << ", Section 1 end\n";
				p = 1;
			}
			#pragma omp section
			{
				#pragma omp critical
				cout << "Thread " << thread << ", Section 2 start\n";
				usleep(1000000);
				#pragma omp critical
				cout << "Thread " << thread << ", Section 2 end\n";
				p = 2;
			}
		}
	    #pragma omp sections
		{
			#pragma omp section
			{
				#pragma omp critical
				cout << "Thread " << thread << ", Section 3 start, p = " << p << "\n";
				usleep(200000);
				#pragma omp critical
				cout << "Thread " << thread << ", Section 3 end\n";
				p = 3;
			}
			#pragma omp section
			{
				#pragma omp critical
				cout << "Thread " << thread << ", Section 4 start, p = " << p << "\n";
				usleep(200000);
				#pragma omp critical
				cout << "Thread " << thread << ", Section 4 end\n";
				p = 4;
			}
			#pragma omp section
			{
				#pragma omp critical
				cout << "Thread " << thread << ", Section 5 start, p = " << p << "\n";
				usleep(200000);
				#pragma omp critical
				cout << "Thread " << thread << ", Section 5 end\n";
				p = 5;
			}
		}
	}

	return 0;
}
